<?php

use YaBandPay\PersiLiao\Cryptography;
use YaBandPay\PersiLiao\Notify;
use YaBandPay\PersiLiao\Payment;
use YaBandPay\PersiLiao\Response;
use YaBandPay\PersiLiao\Api;

/**
 * @project    : PrestaShop
 * @description:
 * @user       : PersiLiao
 * @email      : persi@sixsir.com
 * @date       : 2018/12/28
 * @time       : 22:24
 */
class YabandpayNotifyModuleFrontController extends ModuleFrontController
{
    /**
     * @var Yabandpay $module
     */
    public $module;

    public function postProcess()
    {
        try{
            $fp = fopen(_PS_MODULE_DIR_ . 'yabandpay/Lock.pid','a');
            if(!$fp){
                echo 'fopen error';
                die;
            }
            // 写锁
            flock($fp, LOCK_EX);


            $cryptography = new Cryptography($this->module->getYabandpayToken());
            $notify = new Notify($cryptography);
            $order_info = $notify->getOrderInfo();
            $order_state = Configuration::get('PS_OS_PAYMENT');
            if (isset($order_info['state'])
                && !empty($order_info['state'])
                && $order_info['state'] === Payment::PAY_PAID
            ){
                //创建订单
                $this->module->validateOrder(
                    $order_info['demo']['cart_id'],
                    $order_state,
                    $order_info['demo']['cart_amount'],
                    $order_info['sub_pay_method'],
                    null,
                    array(),
                    $order_info['demo']['currency_id'],
                    false,
                    $order_info['demo']['secure_key']
                );
                $order_id = $this->module->currentOrder;
                Yabandpay::addYabandpayLogTradeId($order_id, $order_info['trade_id']);

                //向YabandPay发送修改order id请求
                $api = new Api(
                    new \YaBandPay\PersiLiao\Account(
                        $this->module->getYabandPayAccount(),
                        $this->module->getYabandPayToken()
                    ),
                    new \YaBandPay\PersiLiao\Request(
                        new Cryptography( $this->module->getYabandPayToken() )
                    )
                );
                $result = $api->updateOrderId($order_info['trade_id'], $order_id);
            }

            // 释放锁
            flock($fp, LOCK_UN);
            fclose($fp);
            Response::ajaxReturnSuccess();
        }catch(\Exception $e){
            PrestaShopLogger::addLog(__CLASS__ . '::' . __FUNCTION__ . '->' . $e->getMessage());
            echo $e->getMessage();
            // Response::ajaxReturnFailure();
        }
    }
}
