<?php

use YaBandPay\PersiLiao\Account;
use YaBandPay\PersiLiao\Api;
use YaBandPay\PersiLiao\Cryptography;
use YaBandPay\PersiLiao\Payment;
use YaBandPay\PersiLiao\Request;

/**
 * @project    : PrestaShop
 * @description:
 * @user       : PersiLiao
 * @email      : persi@sixsir.com
 * @date       : 2018/12/28
 * @time       : 22:24
 */
class YabandpayPaymentModuleFrontController extends ModuleFrontController
{
    /**
     * @var Yabandpay $module
     */
    public $module;

    protected $is_currency_exchange = false;

    public function postProcess()
    {
        $type = Tools::getValue('type');
        if($type !== Payment::PAYMENT_ALIPAY && $type !== Payment::PAYMENT_WECHAT && $type !== Payment::PAYMENT_VISA && $type !== Payment::PAYMENT_MASTERCARD){
            throw new InvalidArgumentException('The action to call is not defined.');
        }
        if($type === Payment::PAYMENT_WECHAT){
            $payment = Payment::WECHAT;
            $order_state = Configuration::get('YABANDPAY_OS_WAITING_WECHATPAY');
        }
        if($type === Payment::PAYMENT_ALIPAY){
            $payment = Payment::ALIPAY;
            $order_state = Configuration::get('YABANDPAY_OS_WAITING_ALIPAY');
        }
        if($type === Payment::PAYMENT_VISA){
            $payment = Payment::VISA;
            $order_state = Configuration::get('YABANDPAY_OS_WAITING_VISA');
        }
        if($type === Payment::PAYMENT_MASTERCARD){
            $payment = Payment::MASTERCARD;
            $order_state = Configuration::get('YABANDPAY_OS_WAITING_MASTERCARD');
        }
        try{
            /**
             * @var Cart $cart
             */
            $cart = $this->context->cart;
            $this->addOrder($cart, $payment, $order_state);
            $api = new Api(new Account($this->module->getYabandPayAccount(), $this->module->getYabandPayToken()), new Request(new Cryptography($this->module->getYabandPayToken())));
            $description = \implode(',', \array_map(function($product){
                if(\is_array($product)){
                    return $product['name'] . '*' . $product['cart_quantity'];
                }
                return $product->name;
            }, $cart->getProducts()));
            $order_id = $cart->id;
            $currency = $this->getCurrency($cart->id_currency);
            if($currency === \YaBandPay\PersiLiao\Currency::CURRENCY_EXCHANGE){
                throw new \InvalidArgumentException('This currency is not supported !');
            }
            if($this->getIsCurrencyExchange() === true){
                $currency = \YaBandPay\PersiLiao\Currency::EUR;
                $order_amount = (string)\YaBandPay\PersiLiao\Currency::calculateToEur($this->getOrderTotal($cart), Configuration::get('YABANDPAY_EXCHANGE_RATE'));
            }else{
                $order_amount = (string)$this->getOrderTotal($cart);
            }

            /* 客户Email */
            if (isset($this->context->customer->email)) {
                $customer_email = $this->context->customer->email;
            } else {
                $customer_email = 'info@yabandpay.com';
            }
            
            $result = $api->payment(
                $payment,
                $order_id,
                $order_amount,
                $currency,
                $description,
                $this->module->getRedirectUrl(),
                $this->module->getNotifyUrl(),
                json_encode([
                    'cart_id'      => $cart->id,
                    'currency_id'  => $cart->id_currency,
                    'secure_key'   => $this->context->customer->secure_key,
                    'redirect_url' => $this->module->getSuccessUrl($cart),
                    'cart_amount'  => (float)$cart->getOrderTotal(true, Cart::BOTH),
                    'plugin_version' => 'prestashop1.7-yabandpay3.3.0',
                ]),
                $customer_email
            );
            if (isset($result['url'])
                && !empty($result['url'])
                && isset($result['trade_id'])
                && !empty($result['trade_id'])
                // && Yabandpay::addYabandpayLogTradeId($order_id, $result['trade_id'])
            ){
                $this->context->cookie->yabandpay_cart_id  = $cart->id;
                $this->context->cookie->yabandpay_trade_id = $result['trade_id'];
                
                Tools::redirect($result['url']);
            }else{
                throw new \InvalidArgumentException('payment api result fail');
            }
        }catch(\Exception $e){
            Tools::redirect('https://partner.yabandpay.com/payments/error?message='.urlencode($e->getMessage()));
        }
    }

    private function getOrderTotal(Cart $cart)
    {
        $amount = $cart->getOrderTotal(true, Cart::BOTH);
        $fee = trim(Configuration::get('YABANDPAY_EXCHANGE_FEE'));
        $fee_amount = 0;
        if($fee && abs($fee) > 0){
            $fee_amount = round($amount * ($fee / 100), 2);
        }
        return $amount + $fee_amount;
    }

    private function addOrder(Cart $cart, $payment_name, $order_state)
    {
        if($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active){
            Tools::redirect('index.php?controller=order&step=1');
        }

        $authorized = false;
        foreach(Module::getPaymentModules() as $module){
            if($module['name'] == Yabandpay::MODULE_NAME){
                $authorized = true;
                break;
            }
        }

        if(!$authorized){
            die($this->trans('This payment method is not available.', array(), 'Modules.Checkpayment.Shop'));
        }

        $customer = new Customer($cart->id_customer);

        if(!Validate::isLoadedObject($customer)){
            Tools::redirect('index.php?controller=order&step=1');
        }

        // $currency = $this->context->currency;
        // $total = (float)$cart->getOrderTotal(true, Cart::BOTH);

        /*
        $this->module->validateOrder(
            (int)$cart->id,
            $order_state,
            $total,
            $payment_name,
            null,
            array(),
            (int)$currency->id,
            false,
            $customer->secure_key
        );
        */

    }

    private function getCurrency($id)
    {
        $currency_id = \intval($id);
        $currencies = $this->module->getCurrency($currency_id);
        if(\is_array($currencies)){
            foreach($currencies as $currency_info){
                if(\intval($currency_info['id_currency']) === $currency_id){
                    $currency = \strtoupper($currency_info['iso_code']);
                    break;
                }
            }
        }elseif($currencies instanceof CurrencyCore){
            $currency = $currencies->iso_code;
        }
        if(isset($currency) && !empty($currency) && \in_array($currency, array( \YaBandPay\PersiLiao\Currency::CNY, \YaBandPay\PersiLiao\Currency::EUR, \trim(\strtoupper(Configuration::get('YABANDPAY_EXCHANGE_CURRENCY'))) ), true)){
            if(\in_array($currency, array( \YaBandPay\PersiLiao\Currency::CNY, \YaBandPay\PersiLiao\Currency::EUR ), true) === false){
                $this->setIsCurrencyExchange(true);
                return \YaBandPay\PersiLiao\Currency::EUR;
            }
            return $currency;
        }
        return \YaBandPay\PersiLiao\Currency::CURRENCY_EXCHANGE;
    }

    /**
     * @return mixed
     */
    public function getIsCurrencyExchange()
    {
        return $this->is_currency_exchange;
    }

    /**
     * @param mixed $is_currency_exchange
     */
    public function setIsCurrencyExchange($is_currency_exchange)
    {
        $this->is_currency_exchange = $is_currency_exchange;
    }


}
