<?php
/**
 * @project    : PrestaShop
 * @description:
 * @user       : PersiLiao
 * @email      : persi@sixsir.com
 * @date       : 2018/12/28
 * @time       : 16:03
 */

if(!defined('_PS_VERSION_')){
    exit;
}

if(\file_exists(__DIR__ . '/vendor/autoload.php') === false){
    exit;
}

require_once __DIR__ . '/vendor/autoload.php';

Class Yabandpay extends PaymentModule
{
    const MODULE_NAME = 'yabandpay';

    public $name = 'yabandpay';

    public $author = 'YaBand';

    public $version = '3.3.0';

    public $display = 'view';

    public $module_key = '9ffe1a467ff3f190cab188b2953c32d8';

    public $controllers = array( 'payment', 'notify' );

    public $ps_versions_compliancy = array( 'min' => '1.5', 'max' => '1.6' );

    public $bootstrap = true;

    public $is_eu_compatible = 1;

    public $is_configurable = 1;

    public $limited_countries = array();

    protected $config = array(
        'YABANDPAY_ACCOUNT',
        'YABANDPAY_TOKEN',
        'YABANDPAY_EXCHANGERATE',
        'YABANDPAY_WECHATPAY',
        'YABANDPAY_ALIPAY',
        'YABANDPAY_VISA',
        'YABANDPAY_MASTERCARD',
        'YABANDPAY_ACTIVE',
        'YABANDPAY_EXCHANGE_RATE',
        'YABANDPAY_EXCHANGE_CURRENCY',
        'YABANDPAY_EXCHANGE_FEE'
    );

    protected $pay_status = array(
        'YABANDPAY_OS_WAITING_WECHATPAY' => array(
            'zh' => '等待微信支付',
            'en' => 'Awaiting for WeChat Pay'
        ),
        'YABANDPAY_OS_WAITING_ALIPAY' => array(
            'zh' => '等待支付宝支付',
            'en' => 'Awaiting for Alipay'
        ),
        'YABANDPAY_OS_WAITING_VISA' => array(
            'zh' => '等待Visa支付',
            'en' => 'Awaiting for Visa'
        ),
        'YABANDPAY_OS_WAITING_MASTERCARD' => array(
            'zh' => '等待Mastercard支付',
            'en' => 'Awaiting for Mastercard'
        ),
    );

    private $tab_modules_list;

    public function __construct()
    {
        parent::__construct();
        $this->tab = 'payments_gateways';
        $this->displayName = 'YaBand Pay';
        $this->description = $this->l('To obtain the login credential, please visit our website www.yabandmedia.com');
        /*if(!count(Currency::checkPaymentCurrencies($this->id))){
            $this->warning = $this->l('No currency has been set for this module.');
        }*/
        $this->confirmUninstall = $this->l('Are you sure to uninstall?');
        $this->tab_modules_list = _PS_ROOT_DIR_ . ModuleCore::CACHE_FILE_TAB_MODULES_LIST;
    }

    public function install()
    {
        // Install default
        if(!parent::install()){
            return false;
        }
        if(!$this->installOrderState()){
            return false;
        }
        // install DataBase
        if(!$this->installSQL()){
            return false;
        }
        // Registration hook
        if(!$this->registrationHook()){
            return false;
        }
        foreach($this->config as $key){
            if(!Configuration::updateValue($key, '')){
                return false;
            }
        }
        if(!$this->installXmlModules()){
            return false;
        }
        return true;
    }

    protected function installXmlModules()
    {
        $obj = \simplexml_load_file($this->tab_modules_list);
        if(!$obj){
            return false;
        }
        foreach($obj->tab as $XMLElement){
            foreach($XMLElement->attributes() as $key => $value){
                $class_value = (array)$value;
                $class_value = $class_value[0];
                if($key === 'class_name' && $class_value === 'AdminPayment'){
                    $modules = $XMLElement->xpath('module');
                    $count = count($modules) + 1;
                    $module = $XMLElement->addChild('module');
                    $module->addAttribute('name', self::MODULE_NAME);
                    $module->addAttribute('position', $count);
                    break 2;
                }
            }
        }
        $obj->saveXML($this->tab_modules_list);
        return true;
    }

    private function installSql()
    {
        $sql_list = array(
            'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'yabandpay_log` (
              `id` bigint(20) NOT NULL AUTO_INCREMENT,
              `order_id` varchar(64) NOT NULL,
              `trade_id` varchar(64) NOT NULL,
              `transcation_id` varchar(64) DEFAULT NULL,
              PRIMARY KEY (`id`),
              UNIQUE KEY `trade_id` (`trade_id`) USING HASH
            ) ENGINE=' . _MYSQL_ENGINE_ . ';'
        );
        foreach($sql_list as $sql){
            if(!Db::getInstance()->execute($sql)){
                return false;
            }
        }
        return true;
    }

    private function installOrderState()
    {
        foreach(\array_keys($this->pay_status) as $pay_status){
            if(!Configuration::get($pay_status)
                || !Validate::isLoadedObject(new OrderState(Configuration::get($pay_status)))){
                $order_state = new OrderState();
                $order_state->name = array();
                foreach(Language::getLanguages() as $language){
                    if(Tools::strtolower($language['iso_code']) == 'zh'){
                        $order_state->name[$language['id_lang']] = $this->pay_status[$pay_status]['zh'];
                    }else{
                        $order_state->name[$language['id_lang']] = $this->pay_status[$pay_status]['en'];
                    }
                }
                $order_state->color = '#4169E1';
                $order_state->hidden = false;
                $order_state->delivery = false;
                $order_state->logable = false;
                $order_state->invoice = false;
                if($order_state->add()){
                    $source = __DIR__ . '/logo.png';
                    $destination = _PS_ROOT_DIR_ . '/img/os/' . (int)$order_state->id . '.gif';
                    \copy($source, $destination);
                }
                Configuration::updateValue($pay_status, (int)$order_state->id);
            }
        }

        return true;
    }

    private function registrationHook()
    {
        if(!$this->registerHook('payment')
            || !$this->registerHook('displayPaymentEU')
            || !$this->registerHook('displayPayment')
            || !$this->registerHook('paymentReturn')
            || !$this->registerHook('displayOrderConfirmation')
            || !$this->registerHook('displayAdminOrder')
            || !$this->registerHook('actionOrderStatusPostUpdate')
            || !$this->registerHook('actionOrderStatusUpdate')
            || !$this->registerHook('header')
            || !$this->registerHook('actionObjectCurrencyAddAfter')
            || !$this->registerHook('displayBackOfficeHeader')
            || !$this->registerHook('displayFooterProduct')
            || !$this->registerHook('actionBeforeCartUpdateQty')
            || !$this->registerHook('displayReassurance')
            || !$this->registerHook('displayInvoiceLegalFreeText')
            || !$this->registerHook('actionAdminControllerSetMedia')
            || !$this->registerHook('displayMyAccountBlock')
            || !$this->registerHook('displayCustomerAccount')
            || !$this->registerHook('displayShoppingCartFooter')
            || !$this->registerHook('actionCartSave')
        ){
            return false;
        }

        return true;
    }

    public function uninstall()
    {
        $this->config[] = 'YABANDPAY_OS_WAITING';
        $this->config = \array_merge($this->config, array_keys($this->pay_status));
        foreach($this->config as $key){
            Configuration::deleteByName($key);
        }

        //Uninstall DataBase
        if(!$this->uninstallSQL()){
            return false;
        }

        // Uninstall default
        if(!parent::uninstall()){
            return false;
        }
        return true;
    }

    private function uninstallSQL()
    {
        $sql_list = array(
            'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'yabandpay_log`'
        );
        foreach($sql_list as $sql){
            if(!DB::getInstance()->execute($sql)){
                return false;
            }
        }
        return true;
    }

    public static function addYabandpayLogTradeId($order_id, $trade_id)
    {
        $sql = 'INSERT INTO ' . _DB_PREFIX_ . 'yabandpay_log (`order_id`,`trade_id`)VALUES(' . $order_id . ',\'' . $trade_id . '\')';
        if(!DB::getInstance()->execute($sql)){
            return false;
        }
        return true;
    }

    public static function updateYabandpayLogTransactionIdByTradeId($trade_id, $transaction_id)
    {
        $sql = 'UPDATE ' . _DB_PREFIX_ . 'yabandpay_log SET `transcation_id`=\'' . $transaction_id . '\' WHERE `trade_id`=\'' . $trade_id . '\'';
        if(!DB::getInstance()->execute($sql)){
            return false;
        }
        return true;
    }

    public function useMobile()
    {
        if((method_exists($this->context, 'getMobileDevice') && $this->context->getMobileDevice())
            || Tools::getValue('ps_mobile_site')){
            return true;
        }

        return false;
    }

    public function hookPayment($params)
    {
        return '';
    }
    public function hookDisplayPayment($params)
    {
        if(!$this->active){
            return false;
        }
        /*if($this->useMobile()){
            return false;
        }*/
        $active = Configuration::get('YABANDPAY_ACTIVE');
        if(empty($active) || !$active){
            return false;
        }
        $options = '<style>.payment_module_yabandpay{padding-left: 17px !important;}.payment_module_yabandpay>span{margin-left: 30px;}</style>';
        $wechatpay = Configuration::get('YABANDPAY_WECHATPAY');
        if($wechatpay){
            /**
             * @var LanguageCore $language
             */
            $language = $this->context->language;
            if($language->language_code === 'zh-cn' || $language->language_code === 'zh-tw'){
                $wechatpay_logo = Media::getMediaPath(_PS_MODULE_DIR_ . '/' . $this->name . '/views/images/yabandpay_wechatpay_zh_100_32.png');
            }else{
                $wechatpay_logo = Media::getMediaPath(_PS_MODULE_DIR_ . '/' . $this->name . '/views/images/yabandpay_wechatpay_en_100_32.png');
            }
            $options .= '<div class="row"><div class="col-xs-12 col-md-6"><p class="payment_module"><a class="payment_module_yabandpay" href="' . $this->context->link->getModuleLink($this->name, 'payment', array( 'type' => 'wechatpay' ), true) . '" title="' . $this->l('Welcome to WeChat Pay') . '"><img width="100" height="32" src="' . $wechatpay_logo . '" alt="' . $this->l('Welcome to WeChat Pay') . '" height="48px"><span>' . $this->l('Welcome to WeChat Pay') . '</span></a></p></div></div>';
        }
        $alipay = Configuration::get('YABANDPAY_ALIPAY');
        if($alipay){
            $options .= '<div class="row"><div class="col-xs-12 col-md-6"><p class="payment_module"><a class="payment_module_yabandpay" href="' . $this->context->link->getModuleLink($this->name, 'payment', array( 'type' => 'alipay' ), true) . '" title="' . $this->l('Welcome to Alipay') . '"><img width="100" height="32" src="' . Media::getMediaPath(_PS_MODULE_DIR_ . '/' . $this->name . '/views/images/yabandpay_alipay_en_100_32.png') . '" alt="' . $this->l('Welcome to Alipay') . '" height="48px"><span>' . $this->l('Welcome to Alipay') . '</span></a></p></div></div>';
        }
        $visa = Configuration::get('YABANDPAY_VISA');
        if($visa){
            $options .= '<div class="row"><div class="col-xs-12 col-md-6"><p class="payment_module"><a class="payment_module_yabandpay" href="' . $this->context->link->getModuleLink($this->name, 'payment', array( 'type' => 'visa' ), true) . '" title="' . $this->l('Welcome to Visa') . '"><img src="' . Media::getMediaPath(_PS_MODULE_DIR_ . '/' . $this->name . '/views/images/yabandpay_visa_en_100_32.png') . '" alt="' . $this->l('Welcome to Visa') . '" height="48px"><span>' . $this->l('Welcome to Visa') . '</span></a></p></div></div>';
        }
        $mastercard = Configuration::get('YABANDPAY_MASTERCARD');
        if($mastercard){
            $options .= '<div class="row"><div class="col-xs-12 col-md-6"><p class="payment_module"><a class="payment_module_yabandpay" href="' . $this->context->link->getModuleLink($this->name, 'payment', array( 'type' => 'mastercard' ), true) . '" title="' . $this->l('Welcome to Mastercard') . '"><img src="' . Media::getMediaPath(_PS_MODULE_DIR_ . '/' . $this->name . '/views/images/yabandpay_mastercard_en_100_32.png') . '" alt="' . $this->l('Welcome to Mastercard') . '" height="48px"><span>' . $this->l('Welcome to Mastercard') . '</span></a></p></div></div>';
        }
        return $options;
    }

    public function hookDisplayOrderConfirmation(array $params)
    {
        if(!$this->active){
            return;
        }
        /**
         * @var Order $order
         */
        if(isset($params['objOrder']) && $params['objOrder'] instanceof Order){
            $order = $params['objOrder'];
        }else{
            $order = $params['order'];
        }
        $order_state = $order->getCurrentState();
        if (\in_array(
            $order_state,
            array(
                Configuration::get('PS_OS_OUTOFSTOCK_UNPAID'),
                Configuration::get('YABANDPAY_OS_WAITING_WECHATPAY'),
                Configuration::get('YABANDPAY_OS_WAITING_ALIPAY'),
                Configuration::get('YABANDPAY_OS_WAITING_VISA'),
                Configuration::get('YABANDPAY_OS_WAITING_MASTERCARD'),
            ),
            true
        )
        ){
            try{
                $result = \str_replace('\\', '', $_POST['resultData']);
                $response = \json_decode($result, true);
                if (\json_last_error()) {
                    throw new \InvalidArgumentException($this->l('Syntax error, malformed JSON.'));
                }
                if (!isset($response['data']['trade_id']) 
                    || empty($response['data']['trade_id'])) {
                    throw new \InvalidArgumentException($this->l('Data error trade_id not found.'));
                }
                $trade_id = $response['data']['trade_id'];
                $cryptography = new \YaBandPay\PersiLiao\Cryptography($this->getYabandpayToken());
                if ($cryptography->verify($response) === false) {
                    throw new \InvalidArgumentException(
                        $this->l(
                            'sign verify error'
                        )
                    );
                }else{
                    $pay_info = $response['data'];
                }
                if (isset($pay_info['state']) 
                    && !empty($pay_info['state']) 
                    && $pay_info['state'] === \YaBandPay\PersiLiao\Payment::PAY_PAID
                ){
                    if (\in_array(
                        $order_state,
                        array(
                            Configuration::get('YABANDPAY_OS_WAITING_WECHATPAY'),
                            Configuration::get('YABANDPAY_OS_WAITING_ALIPAY'),
                            Configuration::get('YABANDPAY_OS_WAITING_VISA'),
                            Configuration::get('YABANDPAY_OS_WAITING_MASTERCARD'),
                        ),
                        true
                    )
                    ) {
                        $order->setCurrentState(
                            Configuration::get('PS_OS_PAYMENT')
                        );         
                    } elseif ($order_state === Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')) {
                        $order->setCurrentState(
                            Configuration::get('PS_OS_OUTOFSTOCK_PAID')
                        );
                    }
                }
            }catch(\Exception $e){
                $error_msg = $e->getMessage();
                PrestaShopLogger::addLog($error_msg);
                return $this->displayError($this->l($error_msg));
            }
        }
    }

    public function hookActionCartSave(array $params)
    {
        $cookie = $this->context->cookie;

        if (!empty($cookie->id_cart)
            && !empty($cookie->yabandpay_cart_id)
            && !empty($cookie->yabandpay_trade_id)
            && $cookie->id_cart == $cookie->yabandpay_cart_id
        ){
            $api = new \YaBandPay\PersiLiao\Api(
                new \YaBandPay\PersiLiao\Account(
                    $this->getYabandPayAccount(),
                    $this->getYabandPayToken()
                ),
                new \YaBandPay\PersiLiao\Request(
                    new \YaBandPay\PersiLiao\Cryptography( $this->getYabandPayToken() )
                )
            );
            $result = $api->cancel($cookie->yabandpay_trade_id);
            unset($cookie->yabandpay_cart_id);
            unset($cookie->yabandpay_trade_id);
        }
    }

    private function _postProcess()
    {
        if(Tools::isSubmit('btnSubmit')){
            $account = Tools::getValue('yabandpay_account');
            $token = Tools::getValue('yabandpay_token');
            if(empty($account) || empty($token)){
                return $this->displayError($this->l('Account or Token can not be empty.'));
            }
            Configuration::updateValue('YABANDPAY_WECHATPAY', Tools::getValue('yabandpay_wechatpay_on'));
            Configuration::updateValue('YABANDPAY_ALIPAY', Tools::getValue('yabandpay_alipay_on'));
            Configuration::updateValue('YABANDPAY_VISA', Tools::getValue('yabandpay_visa_on'));
            Configuration::updateValue('YABANDPAY_MASTERCARD', Tools::getValue('yabandpay_mastercard_on'));
            Configuration::updateValue('YABANDPAY_EXCHANGE_CURRENCY', \trim(Tools::getValue('yabandpay_exchange_currency')));
            Configuration::updateValue('YABANDPAY_EXCHANGE_RATE', Tools::getValue('yabandpay_exchange_rate'));
            Configuration::updateValue('YABANDPAY_EXCHANGE_FEE', Tools::getValue('yabandpay_exchange_fee'));

            Configuration::updateValue('YABANDPAY_ACCOUNT', $account);
            Configuration::updateValue('YABANDPAY_TOKEN', $token);
            try{
                Configuration::updateValue('YABANDPAY_ACTIVE', true);
                return $this->displayConfirmation($this->l('Configuration saved successfully.'));
            }catch(\Exception $e){
                Configuration::updateValue('YABANDPAY_ACTIVE', false);
                return $this->displayError($this->l('Account or Token invalid.'));
            }
        }
        return '';
    }

    public function getContent()
    {
        $html = '';
        $html .= $this->_postProcess();
        $html .= $this->renderForm();
        return $html;
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => 'YaBand Pay',
                    'icon' => 'icon-envelope'
                ),
                'input' => array(
                    array(
                        'type' => 'checkbox',
                        'label' => $this->l(\YaBandPay\PersiLiao\Payment::WECHAT),
                        'name' => 'yabandpay_wechatpay',
                        'values' => array(
                            'query' => array(
                                array( 'id' => 'on', 'name' => $this->l('Enable/Disable ' . \YaBandPay\PersiLiao\Payment::WECHAT), 'val' => '1' ),
                            ),
                            'id' => 'id',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'checkbox',
                        'label' => $this->l(\YaBandPay\PersiLiao\Payment::ALIPAY),
                        'name' => 'yabandpay_alipay',
                        'values' => array(
                            'query' => array(
                                array( 'id' => 'on', 'name' => $this->l('Enable/Disable ' . \YaBandPay\PersiLiao\Payment::ALIPAY), 'val' => '1' ),
                            ),
                            'id' => 'id',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'checkbox',
                        'label' => $this->l(\YaBandPay\PersiLiao\Payment::VISA),
                        'name' => 'yabandpay_visa',
                        'values' => array(
                            'query' => array(
                                array( 'id' => 'on', 'name' => $this->l('Enable/Disable ' . \YaBandPay\PersiLiao\Payment::VISA), 'val' => '1' ),
                            ),
                            'id' => 'id',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'checkbox',
                        'label' => $this->l(\YaBandPay\PersiLiao\Payment::MASTERCARD),
                        'name' => 'yabandpay_mastercard',
                        'values' => array(
                            'query' => array(
                                array( 'id' => 'on', 'name' => $this->l('Enable/Disable ' . \YaBandPay\PersiLiao\Payment::MASTERCARD), 'val' => '1' ),
                            ),
                            'id' => 'id',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'text',
                        'label' => 'YaBand Account',
                        'name' => 'yabandpay_account',
                        'class' => '" style="width:500px',
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => 'YaBand Key',
                        'name' => 'yabandpay_token',
                        'required' => true,
                        'class' => '" style="width:500px'
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Currency'),
                        'name' => 'yabandpay_exchange_currency',
                        'desc' => $this->l('You can set a currency which you want to use our payment, except for CNY and Euro.'),
                        'class' => '" style="width:500px',
                        'required' => false
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Euro exchange rate'),
                        'name' => 'yabandpay_exchange_rate',
                        'desc' => $this->l('Please enter the Exchange Rate (1EUR=?)'),
                        'class' => '" style="width:500px',
                        'required' => false
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Fee(%)'),
                        'name' => 'yabandpay_exchange_fee',
                        //'desc' => $this->l(''),
                        'class' => '" style="width:500px',
                        'required' => false
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
        );

        $this->fields_form = array();

        return $helper->generateForm(array( $fields_form ));
    }

    protected function getConfigFieldsValues()
    {
        return array(
            'yabandpay_account' => Tools::getValue('yabandpay_account', Configuration::get('YABANDPAY_ACCOUNT')),
            'yabandpay_token' => Tools::getValue('yabandpay_token', Configuration::get('YABANDPAY_TOKEN')),
            'yabandpay_wechatpay_on' => Tools::getValue('yabandpay_wechatpay_on', Configuration::get('YABANDPAY_WECHATPAY')),
            'yabandpay_alipay_on' => Tools::getValue('yabandpay_alipay_on', Configuration::get('YABANDPAY_ALIPAY')),
            'yabandpay_visa_on' => Tools::getValue('yabandpay_visa_on', Configuration::get('YABANDPAY_VISA')),
            'yabandpay_mastercard_on' => Tools::getValue('yabandpay_mastercard_on', Configuration::get('YABANDPAY_MASTERCARD')),
            'yabandpay_exchange_rate' => Tools::getValue('yabandpay_exchange_rate', Configuration::get('YABANDPAY_EXCHANGE_RATE')),
            'yabandpay_exchange_currency' => Tools::getValue('yabandpay_exchange_currency', Configuration::get('YABANDPAY_EXCHANGE_CURRENCY')),
            'yabandpay_exchange_fee' => Tools::getValue('yabandpay_exchange_fee', Configuration::get('YABANDPAY_EXCHANGE_FEE'))
        );
    }

    public function getYabandPayAccount()
    {
        return Configuration::get('YABANDPAY_ACCOUNT');
    }

    public function getYabandPayToken()
    {
        return Configuration::get('YABANDPAY_TOKEN');
    }

    public function getUrl($controller, array $params = array())
    {
        return $this->context->link->getModuleLink($this->name, $controller, $params, true);
    }

    public function getNotifyUrl()
    {
        return $this->getUrl('notify');
    }

    public function getRedirectUrl()
    {
        return $this->getUrl('redirect');
    }

    public function getSuccessUrl(Cart $cart)
    {
        return $this->context->link->getPageLink('order-confirmation', true, (int)Context::getContext()->language->id, array( 'id_cart' => $cart->id, 'id_module' => $this->id, 'id_order' => $this->currentOrder, 'key' => (new Customer($cart->id_customer))->secure_key ));
    }
}
